/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.uinfo.salix;

import java.util.HashSet;
import java.util.Set;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.common.AbstractProcessor;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.artifacts.uinfo.common.UInfoResultType;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.process.WOutProcessor;
import org.dive4elements.river.themes.ThemeDocument;

/**
 * Processor to generate the facets and data series of salix line
 *
 * @author Matthias Schäfer
 *
 */
public final class SalixLineProcessor extends AbstractProcessor {

    private static final String FACET_SALIX_LINE = "uinfo_facet_salix_line";

    private static final String FACET_SALIX_LINE_DESCRIPTION = "uinfo_facet_salix_line.description";

    private static final String FACET_SALIX_MNWMW = "uinfo_facet_salix_mnwmw";

    private static final String FACET_SALIX_MNWMW_DESCRIPTION = "uinfo_facet_salix_mnwmw.description";

    public static final String FACET_SALIX_SCENARIO = "uinfo_facet_salix_scenario";

    private static final String FACET_SALIX_SCENARIO_DESCRIPTION = "uinfo_facet_salix_scenario.description";

    private static final Set<String> HANDLED_FACET_TYPES = new HashSet<>();

    static {
        HANDLED_FACET_TYPES.add(FACET_SALIX_LINE);
        HANDLED_FACET_TYPES.add(FACET_SALIX_MNWMW);
        HANDLED_FACET_TYPES.add(FACET_SALIX_SCENARIO);
    }

    public static Facet createSalixLineFacet(final CallContext context, final String hash, final String id, final int facetIndex, final int resultIndex) {

        final String description = Resources.getMsg(context.getMeta(), FACET_SALIX_LINE_DESCRIPTION, FACET_SALIX_LINE_DESCRIPTION);
        return new SalixLineResultFacet(facetIndex, resultIndex, FACET_SALIX_LINE, description, WOutProcessor.I18N_AXIS_LABEL, id, hash);
    }

    public static Facet createSalixMnwMwFacet(final CallContext context, final String hash, final String id, final int facetIndex, final int resultIndex) {

        final String description = Resources.getMsg(context.getMeta(), FACET_SALIX_MNWMW_DESCRIPTION, FACET_SALIX_MNWMW_DESCRIPTION);
        return new SalixMnwMwResultFacet(facetIndex, resultIndex, FACET_SALIX_MNWMW, description, WOutProcessor.I18N_AXIS_LABEL, id, hash);
    }

    public static Facet createSalixScenarioFacet(final CallContext context, final String hash, final String id, final int dataIndex, final int resultIndex,
            final int facetIndex, final String subLabel) {

        final String description = Resources.getMsg(context.getMeta(), FACET_SALIX_SCENARIO_DESCRIPTION, FACET_SALIX_SCENARIO_DESCRIPTION, subLabel);
        return new SalixScenarioResultFacet(facetIndex, resultIndex, dataIndex, FACET_SALIX_SCENARIO, description, WOutProcessor.I18N_AXIS_LABEL, hash, id);
    }

    public SalixLineProcessor() {
        super(WOutProcessor.I18N_AXIS_LABEL, HANDLED_FACET_TYPES);
    }

    @Override
    protected String generateSeries(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible) {

        // log.debug("Processing facet: " + bundle.getFacetName());
        final CallContext context = generator.getContext();
        final Object data = bundle.getData(context);

        if (bundle.getFacetName().equals(FACET_SALIX_LINE)) {
            return buildSeriesForType(generator, bundle, theme, visible, UInfoResultType.salixline, GAP_DISTANCE);
        }

        if (bundle.getFacetName().equals(FACET_SALIX_MNWMW)) {
            return buildSeriesForType(generator, bundle, theme, visible, UInfoResultType.salix_delta_mw, GAP_DISTANCE);
        }

        if (bundle.getFacetName().equals(FACET_SALIX_SCENARIO)) {
            // TODO Differenzieren, Scenario 1 bis max. 5 bei Regional

            // FIXME: warum so unterscheiden? bei den anderen beiden szenarien einfach nur ein szenario-ergebnis in die liste
            // packen!

            if (data instanceof SalixLineCalculationRegionalResult) {

                final int dataIndex = ((SalixScenarioResultFacet) bundle.getFacet()).getDataIndex();

                final double[][] scenarioPoints = ((SalixLineCalculationRegionalResult) data).getScenarioPoints(dataIndex);
                return buildSeriesForPoints(scenarioPoints, generator, bundle, theme, visible, GAP_DISTANCE);

            } else
                return buildSeriesForType(generator, bundle, theme, visible, UInfoResultType.salix_line_scenario, GAP_DISTANCE);
        }

        throw new UnsupportedOperationException();
    }
}